@@ -5,7 +5,7 @@ Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
-    https://www.apache.org/licenses/LICENSE-2.0
+    http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
@@ -24,34 +24,36 @@ import (
 	"time"
 
 	// Inject the fakes for informers this reconciler depends on.
+	networkingclient "knative.dev/networking/pkg/client/injection/client/fake"
+	_ "knative.dev/networking/pkg/client/injection/informers/networking/v1alpha1/serverlessservice/fake"
 	kubeclient "knative.dev/pkg/client/injection/kube/client"
 	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
 	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/service/fake"
 	"knative.dev/pkg/logging"
-	servingclient "knative.dev/serving/pkg/client/injection/client/fake"
 	"knative.dev/serving/pkg/client/injection/ducks/autoscaling/v1alpha1/podscalable"
 	_ "knative.dev/serving/pkg/client/injection/ducks/autoscaling/v1alpha1/podscalable/fake"
-	_ "knative.dev/serving/pkg/client/injection/informers/networking/v1alpha1/serverlessservice/fake"
-
-	apierrs "k8s.io/apimachinery/pkg/api/errors"
-	"knative.dev/pkg/configmap"
-	"knative.dev/pkg/controller"
-	"knative.dev/pkg/ptr"
-	"knative.dev/pkg/system"
-	"knative.dev/serving/pkg/apis/networking"
-	nv1a1 "knative.dev/serving/pkg/apis/networking/v1alpha1"
-	v1 "knative.dev/serving/pkg/apis/serving/v1"
-	sksreconciler "knative.dev/serving/pkg/client/injection/reconciler/networking/v1alpha1/serverlessservice"
-	"knative.dev/serving/pkg/reconciler/serverlessservice/resources"
-	presources "knative.dev/serving/pkg/resources"
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
+	apierrs "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	clientgotesting "k8s.io/client-go/testing"
+	"k8s.io/client-go/tools/cache"
+
+	pkgnet "knative.dev/networking/pkg/apis/networking"
+	nv1a1 "knative.dev/networking/pkg/apis/networking/v1alpha1"
+	sksreconciler "knative.dev/networking/pkg/client/injection/reconciler/networking/v1alpha1/serverlessservice"
+	"knative.dev/pkg/configmap"
+	"knative.dev/pkg/controller"
+	"knative.dev/pkg/ptr"
+	"knative.dev/pkg/system"
+	v1 "knative.dev/serving/pkg/apis/serving/v1"
+	"knative.dev/serving/pkg/networking"
+	"knative.dev/serving/pkg/reconciler/serverlessservice/resources"
+	presources "knative.dev/serving/pkg/resources"
 
 	. "knative.dev/pkg/reconciler/testing"
 	. "knative.dev/serving/pkg/reconciler/testing/v1"
@@ -185,34 +187,6 @@ func TestReconcile(t *testing.T) {
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: endpointspub("steady", "to-proxy", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		}},
-	}, {
-		Name: "many-private-services",
-		Key:  "many/privates",
-		Objects: []runtime.Object{
-			SKS("many", "privates", markHappy, WithPubService, WithPrivateService,
-				WithDeployRef("bar")),
-			deploy("many", "bar"),
-			svcpub("many", "privates"),
-			svcpriv("many", "privates"),
-			svcpriv("many", "privates", svcWithName("privates-brutality-is-here")),
-			svcpriv("many", "privates", svcWithName("privates-uncharacteristically-pretty"),
-				WithK8sSvcOwnersRemoved), // unowned, should remain.
-			endpointspub("many", "privates", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("many", "privates", WithSubsets),
-			activatorEndpoints(WithSubsets),
-		},
-		WantDeletes: []clientgotesting.DeleteActionImpl{{
-			ActionImpl: clientgotesting.ActionImpl{
-				Namespace: "many",
-				Verb:      "delete",
-				Resource: schema.GroupVersionResource{
-					Group:    "core",
-					Version:  "v1",
-					Resource: "services",
-				},
-			},
-			Name: "privates-brutality-is-here",
-		}},
 	}, {
 		Name: "user changes public svc",
 		Key:  "public/svc-change",
@@ -690,15 +664,19 @@ func TestReconcile(t *testing.T) {
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
 		retryAttempted = false
 		ctx = podscalable.WithDuck(ctx)
+		psInformerFactory := podscalable.Get(ctx)
 
 		r := &reconciler{
-			kubeclient:        kubeclient.Get(ctx),
-			serviceLister:     listers.GetK8sServiceLister(),
-			endpointsLister:   listers.GetEndpointsLister(),
-			psInformerFactory: podscalable.Get(ctx),
+			kubeclient:      kubeclient.Get(ctx),
+			serviceLister:   listers.GetK8sServiceLister(),
+			endpointsLister: listers.GetEndpointsLister(),
+			listerFactory: func(gvr schema.GroupVersionResource) (cache.GenericLister, error) {
+				_, l, err := psInformerFactory.Get(ctx, gvr)
+				return l, err
+			},
 		}
 
-		return sksreconciler.NewReconciler(ctx, logging.FromContext(ctx), servingclient.Get(ctx),
+		return sksreconciler.NewReconciler(ctx, logging.FromContext(ctx), networkingclient.Get(ctx),
 			listers.GetServerlessServiceLister(), controller.GetEventRecorder(ctx), r)
 	}))
 }
@@ -760,7 +738,7 @@ func markNoEndpoints(sks *nv1a1.ServerlessService) {
 }
 
 func withHTTP2Protocol(sks *nv1a1.ServerlessService) {
-	sks.Spec.ProtocolType = networking.ProtocolH2C
+	sks.Spec.ProtocolType = pkgnet.ProtocolH2C
 }
 
 type deploymentOption func(*appsv1.Deployment)
@@ -790,10 +768,14 @@ func deploy(namespace, name string, opts ...deploymentOption) *appsv1.Deployment
 func withHTTP2Priv(svc *corev1.Service) {
 	svc.Spec.Ports[0].Name = "http2"
 	svc.Spec.Ports[0].TargetPort = intstr.FromInt(networking.BackendHTTP2Port)
+
+	svc.Spec.Ports[4].Name = "http2-istio"
+	svc.Spec.Ports[4].Port = networking.BackendHTTP2Port
+	svc.Spec.Ports[4].TargetPort = intstr.FromInt(networking.BackendHTTP2Port)
 }
 
 func withHTTP2(svc *corev1.Service) {
-	svc.Spec.Ports[0].Port = networking.ServiceHTTP2Port
+	svc.Spec.Ports[0].Port = pkgnet.ServiceHTTP2Port
 	svc.Spec.Ports[0].Name = "http2"
 	svc.Spec.Ports[0].TargetPort = intstr.FromInt(networking.BackendHTTP2Port)
 }
@@ -819,13 +801,6 @@ func svcpub(namespace, name string, so ...K8sServiceOption) *corev1.Service {
 	return s
 }
 
-func svcWithName(n string) K8sServiceOption {
-	return func(s *corev1.Service) {
-		s.GenerateName = ""
-		s.Name = n
-	}
-}
-
 func svcpriv(namespace, name string, so ...K8sServiceOption) *corev1.Service {
 	sks := SKS(namespace, name)
 	s := resources.MakePrivateService(sks, map[string]string{
@@ -932,7 +907,7 @@ func TestSubsetEndpoints(t *testing.T) {
 	})
 	t.Run("actual subset", func(t *testing.T) {
 		// We need to verify two things
-		// 1. that exacly N items were returned
+		// 1. that exactly N items were returned
 		// 2. they are distinct
 		// 3. No empty subset is returned.
 		tests := []struct {
